;
; ideserv.asm - X1541 pclink handler
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
PCLINK_MAGIC	= $af

pclink_init	.proc
		lda #%11000111
		and $dd00
		sta $dd00

		bit $dd00
		bvs *-3

		lda #63-8
		sta $dd02

		bit $dd00
		bvc *-3

		lda #63
		sta $dd02
		rts
		.pend

pclink_send_all	.proc
		cmp #PCLINK_MAGIC
		bne pclink_send
		jsr pclink_send
		lda #0
		geq pclink_sen

pclink_send	.proc
		jsr *+3
		jsr *+3

pclink_sen	.proc
		bit $dd00
		bvs *-3

		asl

		ldy #63-8
		bcc *+4
		ldy #63-8-32
		sty $dd02

		bit $dd00
		bvc *-3

		asl

		ldy #63
		bcc *+4
		ldy #63-32
		sty $dd02
		rts
		.pend

pclink_recieve_all .proc
		lda #$fe
pclink_rec	.proc
		bit $dd00
		bvs *-3

		ldx $dd00
		cpx #$80

		ldx #63-8
		stx $dd02
		rol

		bit $dd00
		bvc *-3

		ldx $dd00
		cpx #$80

		ldx #63
		stx $dd02
		rol
		bcs pclink_rec
		cmp #PCLINK_MAGIC
		beq xeu
		clc
		rts

xeu		lda #0
		jsr pclink_rec
		cmp #1
		lda #PCLINK_MAGIC
		rts
		.pend

pclink_load	.proc
		lda ActualFNLength
		beq exit_missing_filename

		jsr OP_save_zeropage
		jsr OP_copy_filename_to_path
		jsr OP_load_searching_for
                jsr OP_loadautos
		jsr load_install_write_verify_byte
                lda #$ea		;nop
                sta start_zeropage+3

		lda #"L"
		jsr pclink_send_path

		cmp #0
		bne pclink_loaderr

		jsr pclink_recieve_all
		bcs kyp2
		sta $ae
		jsr pclink_recieve_all
		bcs kyp2
		ldx ActualSecAddr	;sec addr!=0?
		bne jklj
		lda $c3
		sta $ae
		lda $c4
jklj		sta $af

		jsr OP_load_loading

		lda $ae
                sta start_zeropage+5
		lda $af
                sta start_zeropage+6
		#jmps oeuqp

eiqj		ldy #0
		jsr start_zeropage
		inc start_zeropage+5
                bne oeuqp
                inc start_zeropage+6
		beq pclink_loaderr2
oeuqp		jsr pclink_recieve_all
		bcc eiqj
                lda #0
                sta $ae
		ldy start_zeropage+5
		lda start_zeropage+6
                sta $af
kyp		#jmps load_end

kyp2		lda #$02
		ora Status
		sta Status
		gne kyp
		.pend

pclink_loaderr	.proc
		jsr OP_restore_zeropage
		#jmps exit_file_not_found
		.pend

pclink_loaderr2	.proc
		jsr OP_restore_zeropage
		#jmps exit_out_of_memory
		.pend

peek_r		.proc
		.logical start_zeropage
		sta cfg_stnd		;3
at1		lda $ff00,x		;3
		sta cfg_open		;3
		rts			;1
		.here
		.pend

pclink_save	.proc
		jsr OP_save_zeropage
		ldx #9
iukq		lda peek_r,x
		sta start_zeropage,x
		dex
		bpl iukq
		jsr OP_copy_filename_to_path

		lda #"S"
		jsr pclink_send_path

		cmp #0
		bne ixq

		jsr OP_save_saving

		#ldatax $c1
		jsr pclink_send_all
		lda $c2
		sta peek_r.at1+2
		jsr pclink_send_all

htj		cpx $ae
		lda peek_r.at1+2
		sbc $af
		bcs xjqxu
		jsr start_zeropage
		jsr pclink_send_all
		inx
		bne htj
		inc peek_r.at1+2
		#jmps htj

xjqxu		stx $ac
		lda peek_r.at1+2
		sta $ad
		lda #PCLINK_MAGIC
		jsr pclink_send
		lda #64
		jsr pclink_sen
ixq		jsr OP_restore_zeropage
		clc
		jmp exit2basic
		.pend

pclink_send_path .proc
		pha
		jsr pclink_init
		pla
		jsr pclink_send
		ldx #0
exq		lda path_exp,x
		beq keuo
		jsr pclink_send
		inx
		bne exq
		txa			;should not happen
keuo		jsr pclink_send
		#jmps pclink_recieve_all
		.pend

pclink_open	jmp *
