;
; read.asm - BLKREAD kernal extension handler
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;**************************************************
; Read (A=start address on zeropage, X,Y number of bytes to read)
;
; Used registers: A X Y (X,Y=Number of bytes read)
; Status: C
; C=1:
; A=5 device not present
; A=6 not input file
; A=9 illegal device number
; Status response:
; ST=$00 ok
; ST=$40 end of file
; ST=$42 truncated
; ST=$52 readerror
; ST=$80 device not present
;**************************************************
;----------------------------------------
;IDE readerr
;
read_exerr	.proc
		pha
		jsr OP_restore_zeropage
		pla
		cmp #INFO_KERNAL
		beq exit_illegal_device
		cmp #INFO_BLK_WRITE
		beq hrh
		cmp #INFO_BLK_WRITE_FULL
		beq hrh
;		cmp #INFO_CHAR_READ
;		beq hrh
;		cmp #INFO_CHAR_RW
;		beq hrh
;		cmp #INFO_CHAR_WRITE
;		beq hrh
		cmp #INFO_STATUS
		beq hrh

		lda #$42
		sta Status
		ldx #0
		ldy #0
		clc
		jmp exit2basic

hrh		jmp exit_not_input_file
		.pend
;----------------------------------------
;IDE read
;
newread		.proc
		pha
		#stx_l_a read_size_exp
		#stx_l_a read_all_exp
		#sty_l_a read_size_exp+1
		#sty_l_a read_all_exp+1

		jsr OP_save_zeropage

		#plx_a
		ldy $00,x
		lda $01,x		;get destination address
                sty pom4
                sta pom5

		#ldatax buffer_input_exp
		sta buffer_currentp_exp
		lda buffer_info_exp,x
		bmi read_exerr
		lda buffer_type_exp,x
		cmp #BUFFERMODE_FILE
		beq read_file
		jsr OP_restore_zeropage
		jmp exit_not_input_file
                
read_file	ldy #0
                tsx
fg              cpx #$02+44		;enough stack space?
                bcs ook
                pla
                sta sp_exp,x		;no, let's backup
                iny
                inx
                gne fg
ook		#sty_l_a sp_deep_exp

		jsr XIT_read_file
                sta help_x_exp

                #ldy_l_a sp_deep_exp
                beq kb
                ldx #$02+44
nk              dex
		lda sp_exp,x         ;restore stack
                pha
                dey
                bne nk
kb
                lda help_x_exp
		bcc *+4
                lda #$52
                sta Status
                php

		jsr OP_restore_zeropage
                
		lda read_all_exp
		sec
		sbc read_size_exp
		tax
		lda read_all_exp+1
		sbc read_size_exp+1
		tay

                plp
                lda #$18		;file data error 
                jmp exit2basic
;----------------------------------------
;file read
;
XIT_read_file	#xit_init

		#ldatax buffer_currentp_exp
		lda buffer_info_exp,x
		pha
		lda #INFO_IGNORE	;be safe
		sta buffer_info_exp,x

		#ldy_l_x_a buffer_address_end_hi_exp,x
                beq middle		;no need to sweep out buffer first

                lda buffer_address_low_exp,x
                clc
                adc #1
                sta pom1
                lda buffer_address_hi_exp,x
                and #$7f
                adc #0
                sta pom2

                lda buffer_start,x
                clc
                adc #2
                eor buffer_address_end_hi_exp,x
                cmp #1

		lda buffer_address_end_low_exp,x
		sbc buffer_address_low_exp,x		;How many bytes left in buffer?
                sta pom6
		tya
                sbc buffer_address_hi_exp,x
                and #7
		sta pom7

		#ldatax read_size_exp
		cpx pom6
		lda read_size_exp+1
		sbc pom7
                php			;remember that we requested less bytes than in buffer
		bcs cf
		stx pom6
		lda read_size_exp+1	;Have to read less bytes than in buffer!
		sta pom7

cf		ldx #8
le	     	lda read_wr_one_r,x
                sta start_zeropage,x
                dex
                bpl le

		.if SUPERCPU=1
		ldy #1
		sty pom3
                dey
		.else
                ldy #0
                .fi

		ldx pom7
		beq ty
					;do copy
ug		#lda_zp_y pom1
		jsr start_zeropage
		iny
		bne ug
		inc pom2
		inc pom5
		dex
		bne ug2

ty		ldx pom6
		beq nul

ug2		#lda_zp_y pom1
		jsr start_zeropage
		iny
		dex
		bne ug2

nul		
		#ldatax buffer_currentp_exp
		lda read_size_exp
		sec
		sbc pom6
		sta read_size_exp	;substract copyend bytes
		lda read_size_exp+1
		sbc pom7
		sta read_size_exp+1

		tya
		clc
		adc pom4
		sta pom4		;adjust destination address
                bcc *+4
		inc pom5

		tya
                clc
		adc buffer_address_low_exp,x
		sta buffer_address_low_exp,x;adjust buffer position
		lda buffer_address_hi_exp,x
		and #$7f
		adc pom7
                cmp buffer_address_end_hi_exp,x
		bne *+4
                ora #$80		;in last bank!
                sta buffer_address_hi_exp,x
                plp
		and #$80
                bpl nlst
		lda buffer_address_low_exp,x
                eor buffer_address_end_low_exp,x
                bne nlst		;we read all bytes from buffer

                lda read_size_exp
                ora read_size_exp+1
                beq vw

                ldy #$42		;truncate
vsr             pla
                lda #INFO_BLK_R_EOF
vfe             sta buffer_info_exp,x
                tya
                clc
                rts

vw              ldy #$40		;last byte read
                lda read_all_exp
                ora read_all_exp+1
                bne vsr			;eof
                pla
                jmp vfe			;eof, but last byte still readable

nlst            ldy #0
		bcc vegeok		;we read less bytes than in buffer, so we are ready

		tya
		sta buffer_address_end_hi_exp,x
		sta buffer_address_end_low_exp,x
		lda buffer_start,x	;mark it as unloaded sector
                sec
		sbc #1
		sta buffer_address_hi_exp,x;buffer_address_low_exp is already at 255

		lda buffer_filepos2_exp,x
		adc #2-1
		sta buffer_filepos2_exp,x
		bcc middlex
		#inc_l_x_a buffer_filepos3_exp
		bne middlex
		#inc_l_x_a buffer_filepos4_exp;!!!!!!!!!!
                gcs middlex
;
;	Now we have to load full sectors to (read_lot_cfs_r.at4), buffer is empty
;
middle		lda #0
		sta pom6
		sta pom7

middlex		lda buffer_filesize1_exp,x
                sta pom1
                lda buffer_filesize2_exp,x
                sec
                sbc buffer_filepos2_exp,x;How much bytes till end of file?
                sta pom2
                lda buffer_filesize3_exp,x
                sbc buffer_filepos3_exp,x
                bne enough
                lda buffer_filesize4_exp,x
                sbc buffer_filepos4_exp,x
                bne enough
                
                lda read_size_exp
                cmp pom1
                lda read_size_exp+1
                sbc pom2
                bcc enough
                bne vy
                lda pom1
                eor read_size_exp
vy              php
                lda pom2
                sta read_size_exp+1
                lda pom1		;file shorter than expected
                sta read_size_exp
                clc
                adc pom6
                sta read_all_exp
                lda pom2
                adc pom7
                sta read_all_exp+1
                .byte SKIP2
enough          clc
		php

		lda read_size_exp
                ora read_size_exp+1
                beq vege2

		ldx #41
dt		lda read_lot_cfs_r,x
		pha
		dex
		bpl dt
                
                tsx
                lda pom4
		sta read_lot_cfs_r.at1+1,x
		sta read_lot_cfs_r.at3+1,x
		clc
		adc #$80
		sta read_lot_cfs_r.at2+1,x;init pointers
		sta read_lot_cfs_r.at4+1,x
		lda pom5
		sta read_lot_cfs_r.at1+2,x
		adc #0
		sta read_lot_cfs_r.at2+2,x
		adc #1
		sta read_lot_cfs_r.at4+2,x
		lda pom5
		adc #1
		sta read_lot_cfs_r.at3+2,x

tnt2		jsr XIT_load_tree	;-=> error

		#ldatax buffer_currentp_exp
		lda buffer_filepos2_exp,x
tnt		lsr
		sta read_p_exp

		lda buffer_start,x
		ora #>b_cfs_file_tree_exp
		tay
		jsr linearaccess

		jsr XIT_upload_next_sector;-=> error

		#ldy_l_a read_sectors_exp
		jsr OP_read
                bcs handle_error

		#ldatax buffer_currentp_exp
		lda status_g
		lda low_reg
read_loop	and #%00001000
		beq handle_error2	;DRQ=1?

		lda read_size_exp+1
		sec
		sbc #2
		bcc lessthan512
		sta read_size_exp+1
                
		ldy #0
                tsx
                lda #1
                pha
                #phx_a
                rts

back		lda read_lot_cfs_r.at1+2,x
		adc #2-1
		sta read_lot_cfs_r.at1+2,x
		adc #1
		sta read_lot_cfs_r.at3+2,x
		lda read_lot_cfs_r.at2+2,x
		adc #2
		sta read_lot_cfs_r.at2+2,x
		adc #1
		sta read_lot_cfs_r.at4+2,x

		#ldatax buffer_currentp_exp
		lda buffer_filepos2_exp,x
		adc #2
		sta buffer_filepos2_exp,x
		bcc fw
		#inc_l_x_a buffer_filepos3_exp
		bne fw
		#inc_l_x_a buffer_filepos4_exp;!!!!!
fw
		lda status_g
		lda low_reg
		bpl okbsy
		jsr wait_busy
		bcs handle_error
		#ldatax buffer_currentp_exp
		lda status_g
		lda low_reg
okbsy
		#dec_l read_sectors_exp
		bne read_loop

		and #%00001000
		bne handle_error2	;DRQ=0?

		lda read_size_exp
		ora read_size_exp+1	;eof!!!!!!!!!!!!!!!!!!!!!!!
		beq vege

		lda buffer_filepos2_exp,x
		bne tnt			;end of a tree sector?
		geq tnt2

handle_error2	sec
		lda #EUNEXP
handle_error	jmp LWOP_reporterror_read_xit;-=> error
;
; Now have to load the rest to (read_lot_cfs_r.at1)
;
lessthan512	lda buffer_start,x
		pha
		tax
		jsr read_buffer_cfs
		jsr ata_pi_nomore_data
		jsr LWOP_reporterror_read_xit;-=> error
                
                pla
                sta pom2

		tsx
		lda read_lot_cfs_r.at1+1,x
                sta pom4
		lda read_lot_cfs_r.at1+2,x
                sta pom5

		.if SUPERCPU=1
		ldy #1
		sty pom3
		dey
		.else
		ldy #0
		.fi
		sty pom1

		ldx #8
le2	     	lda read_wr_one_r,x
                sta start_zeropage,x
                dex
                bpl le2

		#ldatax read_size_exp+1
		beq rl

rl2		#lda_zp_y pom1
		jsr start_zeropage
		iny
		bne rl2
		inc pom2
		inc pom5
		dex
		bne rl2

rl		#ldatax read_size_exp
		beq rl3

cryf2		#lda_zp_y pom1
		jsr start_zeropage
		iny
		dex
		bne cryf2
rl3

		#ldatax buffer_currentp_exp
                tya
                bne *+4
                dec pom2
                dey

		tya
                sta buffer_address_low_exp,x
                ldy pom2

		lda buffer_start,x
                clc
		adc #2
		sta buffer_address_end_hi_exp,x
		lda #0
		sta buffer_address_end_low_exp,x
		sta read_size_exp
		sta read_size_exp+1

		lda buffer_filesize1_exp,x
		sec
		sbc #1
		sta pom1
		lda buffer_filesize2_exp,x
		sbc buffer_filepos2_exp,x
		sta pom2
		lda buffer_filesize3_exp,x
		sbc buffer_filepos3_exp,x
		sta pom3
		lda buffer_filesize4_exp,x
		sbc buffer_filepos4_exp,x
		ora pom3
		bne m512
		lsr pom2
		bne m512

		lda buffer_start,x
		adc #0
		sta buffer_address_end_hi_exp,x
		lda pom1
		sta buffer_address_end_low_exp,x
m512
		tya
                cmp buffer_address_end_hi_exp,x
                bne *+4
                ora #$80
                sta buffer_address_hi_exp,x

vege	      	tsx
		txa
                clc
                adc #42
                tax
                txs

vege2		ldy #0
		#ldatax buffer_currentp_exp
                plp
                bcc vegeok
                beq nt
                ldy #$42
                .byte SKIP2
nt              ldy #$40
                pla
                lda #INFO_BLK_R_EOF
                .byte SKIP1
vegeok		pla			;be safe
		sta buffer_info_exp,x

                tya
		clc
                rts			;-=> ok
		.pend
;-----------------------------------
; CFS reading 512 bytes DUPLICATED FROM LOAD!
;
read_lot_cfs_r	.proc
		.logical $0101
		sta cfg_stnd		;3
lp		lda data_g		;6

		lda low_reg		;9
at1		sta $0000,y		;12

		lda hig_reg		;15
at2		sta $0000,y		;18

		lda data_g		;21

		lda low_reg		;24
at3		sta $0000,y		;27

		lda hig_reg		;30
at4		sta $0000,y		;33
		iny			;34

		bpl lp			;36
		sta cfg_open		;39
		jmp newread.back  	;42
		.here
		.pend

read_wr_one_r	.proc
		.logical start_zeropage
		sta cfg_stnd		;3
		sta (pom4),y
                sta cfg_open		;3
                rts
                .here
                .pend
