;
; save.asm - SAVE kernal extension handler
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;**************************************************
; Save (A=start address on zeropage, X,Y last not saved byte)
;
; Used registers: A X Y
; Zeropage: $c1,$c2=start address
; Zeropage: $ae,$af=X and Y register before call
; Zeropage: $ac,$ad=the byte after the lastsaved
; Status: C
; C=1:
; A=0 break
; A=5 device not present
; A=8 missing filename
; A=9 illegal device number
; A=24 file data
; Status response:
; ST=$00 ok
; ST=$03 timeout
; ST=$80 device not present
;
; Notes:
; Can only save 65533 bytes+load address (1 tree sector)
;**************************************************
;--------------------------------------
; SAVE
;
;		stx $ae
;		sty $af
;		tax
;		lda $00,x
;		sta $c1
;		lda $01,x
;		sta $c2
;		jmp ($0332)
;--------------------------------------
newsave		.proc
		lda ActualDev
		#devicehandler hard_save,pclink_save,exit_illegal_device,kernal_save
		.pend

kernal_save	.proc
		tya
		beq ill			;drive 0 = keyboard
		cmp #31
		bcs ill			;illegal serial device
		cmp #3
		beq ill			;drive 3 = screen
		bcs ok
		#psr $f5f8
		tya
		jmp exit2basic		;Normal kernal rutin
ill		jmp exit_illegal_device

ok
		ldy ActualFNLength
		beq exit_missing_filename

		sta save_p_exp		;store drive number
                pha
                #OP_scpu01up
		jsr OP_save_zeropage
		jsr OP_copy_filename_to_path

		lda #$61
		sta ActualSecAddr
		lda #0
		sta Status
		jsr OP_cfg16up
                pla
                tay
		.if SERIAL_HANG_FIX=1
                jsr test_hang
                bcc np
                .fi
		jsr ieclisten		;listen
		lda #$f1
		jsr iecsecond		;second
		lda Status
		bpl oe
np		jsr OP_cfg16down
		jsr OP_restore_zeropage
                #OP_scpu01down
		jmp exit_device_not_present
                
oe		
		ldx #0
cy		#sta_cfg_open
		lda path_exp,x
		#sta_cfg_16
		beq oy
		jsr iecout		;ciout
		inx
		bne cy
oy
		jsr iecunlisten		;unlsn

		tya
		jsr ieclisten		;listen
		lda #$61
		jsr iecsecond		;second
		lda $c1
		sta $ac
		jsr iecout
		lda $c2
		sta $ad
		jsr iecout
		jsr OP_cfg16down

		jsr OP_save_saving

		ldx #9
di		lda read_byte_r3,x
		sta start_zeropage,x
		dex
		bpl di

		ldy #0
loop		lda $ac
		cmp $ae
		lda $ad
		sbc $af
		bcs ende
		jsr start_zeropage
		tax
		jsr OP_cfg16up
		txa
		jsr iecout
		jsr OP_cfg16down
		lda Status
		bne err
		inc $ac
		bne loop
		jsr check_stop
		beq sto
		inc $ad
		gne loop
err		iny
sto		iny
ende		
		lda save_p_exp
                pha
		jsr OP_cfg16up
		jsr iecunlisten		;unlisten
                pla
                jsr ieclisten		;f648
                lda #$e1
                jsr iecsecond
                jsr iecunlisten
		jsr OP_cfg16down
		jsr OP_restore_zeropage
                #OP_scpu01down
		lda Status
		bne errr
		tya
		beq oke
		dey
errr		bne exit_file_data
		lda #0			;stop pressed
		sec
		jmp exit2basic

oke		jsr OP_load_loading_end
		clc
		jmp exit2basic
		.pend
;--------------------------------------
; SAVE to IDE
;--------------------------------------
hard_save	.proc
		lda ActualFNLength
		beq exit_missing_filename
		jsr OP_save_zeropage
		jsr OP_copy_filename_to_path

                ldy #0
                tsx
fg              cpx #$02+42		;enough stack space?
                bcs ook
                pla
                sta sp_exp,x		;no, let's backup
                iny
                inx
                gne fg
ook		#sty_l_a sp_deep_exp

		jsr XIT_save

                #ldy_l_a sp_deep_exp
                beq kb
                ldx #$02+42
nk              dex
		lda sp_exp,x         ;restore stack
                pha
                dey
                bne nk
kb

	      	jsr OP_restore_zeropage
                bcc bwe
		jmp exit_file_data	;error!
bwe             jmp exit2basic

XIT_save
                #xit_init

		jsr OP_save_saving

                jsr check_drive		;check drive is alive

		jsr XIT_init_drive	;-=> error

		lda #RESOLV_SAVE_MODE
		jsr OP_XIT_resolv_path_part;-=> error
		bcs oknotexists

                lda #$83
		sta Status
                sec
                gcs LWOP_reporterror_63_xit;-=> file exists
                
invl1		.text "DRCDL"
invl2		.text "EEBIN"
invl3		.text "LLMRK"

oknotexists
		ldy #4
ds              lda path_searchtype_exp
                cmp invl1,y
                bne vr
                lda path_searchtype_exp+1
                cmp invl2,y
                bne vr
                lda path_searchtype_exp+2
                cmp invl3,y
                bne vr			;sorry invalid filetype -> prg
                sec
                gcs LWOP_reporterror_64_xit;file type mismatch
vr		dey
		bpl ds

		lda #ram_exp_bitmap
		jsr XIT_prealloc_dir_sector

                jsr find_free_sector	;allocate tree sector

		#cnext next_sector , created_new_tree_sector_exp;remember tree sector position
                
                lda #0
                tax
ksc             sta ram_exp_tree_exp,x	;clear tree sector
                sta ram_exp_tree_exp+$100,x
                inx
                bne ksc
                
                lda $ae
                sec
                sbc $c1
                tax
                lda $af
                sbc $c2
                bcs vmw
                lda #0
                tax
vmw		cpx #254
		inx
                inx
                adc #0
		bcc vxs
                lda #255
                tax
                clc
vxs             sta save_size_exp+1
		sta save_all_exp
                #stx_l_a save_size_exp

		ldx #9
di		lda read_byte_r3,x
		sta start_zeropage,x
		dex
		bpl di

		lda $c1
		sta ram_exp_buffer0b_exp
		sta $ac
		lda $c2
                sta ram_exp_buffer0b_exp+1
		sta $ad
                
                ldy #0
vnsd            jsr start_zeropage
                #sta_l_y_ ram_exp_buffer0b_exp+$002
                iny
                bne vnsd
                inc $ad
                ldy #0
vnsd2           jsr start_zeropage
                #sta_l_y_ ram_exp_buffer0b_exp+$102
                iny
                cpy #254
                bne vnsd2

                lda #0
                sta save_p_exp

                lda #ram_exp_tree
                jsr find_free_sectors_tree

		jsr XIT_upload_next_sector;-=> error
		#ldy_l_a save_sectors_exp;1 sector
		jsr OP_write
		jsr LWOP_reporterror_write_xit;-=> error
		ldx #ram_exp_buffer0b
		jsr write_buffer_cfs

		ldx #41
nvw             lda save_lot_cfs_r,x
                pha
                dex
                bpl nvw
                
                tsx

		lda $c1
                sec
                sbc #2
		sta save_lot_cfs_r.at1+1,x
		sta save_lot_cfs_r.at3+1,x
                sta $ac
		lda $c2
                sbc #0
                sta save_lot_cfs_r.at1+2,x
                lda save_lot_cfs_r.at1+1,x
                clc
                adc #$80
		sta save_lot_cfs_r.at2+1,x
		sta save_lot_cfs_r.at4+1,x
		lda save_lot_cfs_r.at1+2,x
                adc #0
		sta save_lot_cfs_r.at2+2,x
                clc
                gcc back
                
tnt		lda #ram_exp_tree
                jsr find_free_sectors_tree

                jsr XIT_upload_next_sector;-=> error
                #ldy_l_a save_sectors_exp
		jsr OP_write
		bcs handle_error

		lda status_g
		lda low_reg
write_loop	and #%00001000 
		beq handle_error2	;DRQ=1?

		ldy #0
                tsx
                lda #1
                pha
                #phx_a
                rts

back		tsx
		lda save_lot_cfs_r.at1+2,x
		sta $ad
		clc
		adc #2
		sta save_lot_cfs_r.at1+2,x
		adc #1
		sta save_lot_cfs_r.at3+2,x
		lda save_lot_cfs_r.at2+2,x
		adc #2
		sta save_lot_cfs_r.at2+2,x
		adc #1
		sta save_lot_cfs_r.at4+2,x

he2		lda status_g
		#ldatax low_reg
		bpl okbsy
he		jsr wait_busy
                bcc he2
		gcs handle_error

okbsy		ror
		bcs he

		lda save_size_exp+1
		sbc #2-1
		bcc vege2
		sta save_size_exp+1

		txa

		#dec_l save_sectors_exp
		bne write_loop

		and #%00001000 
		bne handle_error2	;DRQ=0?

		lda save_size_exp
		ora save_size_exp+1
                beq vege2
		gne tnt
                
handle_error2	sec
		lda #EUNEXP
handle_error	jmp LWOP_reporterror_write_xit

vege2	        jsr ata_pi_nomore_data
	        jsr LWOP_reporterror_write_xit

vege		lda $ae
		sta $ac
                lda $af
                sta $ad

                #xit_exit		;restore stack

		jsr XIT_find_free_flush	;-=> error
                
                #cnext created_new_tree_sector_exp , next_sector

                lda #ram_exp_tree	;write out the tree sector
		jsr XIT_write_a_sector  ;-=> error

                lda #ram_exp_buffer0
                jsr init_pom1_y

                lda save_size_exp
                sta szam+3
                lda save_all_exp
                sta szam+2
                lda #0
                sta szam+1
                sta szam

                lda #FTYPE_NORMAL | FATTR_EXEC | FATTR_READ | FATTR_WRITE | FATTR_DELETE | FATTR_CLOSED
		jsr XIT_finalize_direntry

		lda #ram_exp_buffer0	;write out the final directory entry
		jsr XIT_write_a_sector  ;-=> error

		jsr LWOP_reporterror_ok

		jsr OP_load_loading_end

		lda #0
                sta Status
		clc
                rts
		.pend
;-----------------------------------
; Read a byte (for save)
;
read_byte_r3	.proc
		.logical start_zeropage
		sta cfg_stnd		;3
		lda ($ac),y		;2
		sta cfg_open		;3
		rts			;1
		.here
		.pend

;-----------------------------------
; CFS saving 512 bytes normal
;
save_lot_cfs_r	.proc
		.logical $101
		sta cfg_stnd		;3
lp		          		

at1		lda $0000,y		;6
		sta low_reg		;9
                
at2		lda $0000,y		;12
		sta hig_reg		;15
                
                sta data_g		;18

at3		lda $0000,y		;21
		sta low_reg		;24
                
at4		lda $0000,y		;27
		sta hig_reg		;30
                
                sta data_g		;33

		iny			;34
		bpl lp			;36
		sta cfg_open		;39
		jmp hard_save.back    	;42
		.here
		.pend
